## Returns the covariance matrix assuming a squared exponential kernel
calcSigmaRBF <- function(X1,X2,tau=1,l=1) {
  tausq <- tau^2
  Sigma <- matrix(rep(0, length(X1)*length(X2)), nrow=length(X1))
  for (i in 1:nrow(Sigma)) {
    for (j in 1:ncol(Sigma)) {
      Sigma[i,j] <- tausq*exp(-1*(abs(X1[i]-X2[j])/l)^2)
    }
  }
  return(Sigma)
}

 
## Returns the covariance between the derivative at X1 and X2 assuming a squared exponential kernel
calcSigmaRBF.x1prime<- function (X1prime, X2, tau, l) {
  tausq <- tau^2
  Sigma <- matrix(rep(0, length(X1prime)*length(X2)), nrow=length(X1prime))
  for (i in 1:nrow(Sigma)) {
    for (j in 1:ncol(Sigma)) {
      Sigma[i,j] <- -2/l^2 * (X1prime[i]-X2[j]) * tausq*exp(-1*(abs(X1prime[i]-X2[j])/l)^2)
    }
  }
  return(Sigma)
}

## Returns the covariance between X1 and the derivative at X2 assuming a squared exponential kernel
calcSigmaRBF.x2prime<- function (X1, X2prime, tau, l) {
  tausq <- tau^2
  Sigma <- matrix(rep(0, length(X1)*length(X2prime)), nrow=length(X1))
  for (i in 1:nrow(Sigma)) {
    for (j in 1:ncol(Sigma)) {
      Sigma[i,j] <- 2/l^2 * (X1[i]-X2prime[j]) * tausq*exp(-1*(abs(X1[i]-X2prime[j])/l)^2)
    }
  }
  return(Sigma)
}

## Returns the covariance between the derivative at X1prime and the derivative at X2prime assuming a squared exponential kernel
calcSigmaRBF.x1x2prime<- function (X1prime, X2prime, tau, l) {
  tausq <- tau^2
  Sigma <- matrix(rep(0, length(X1prime)*length(X2prime)), nrow=length(X1prime))
  for (i in 1:nrow(Sigma)) {
    for (j in 1:ncol(Sigma)) {
      Sigma[i,j] <- (2/l^2 - (X1prime[i]-X2prime[j])^2 * (2/l^2)^2) * tausq*exp(-1*(abs(X1prime[i]-X2prime[j])/l)^2) 
    }
  }
  return(Sigma)
}



## Returns the covariance matrix assuming a linear kernel
calcSigmaLinear <- function(X1,X2) {
  Sigma <- matrix(rep(0, length(X1)*length(X2)), nrow=length(X1))
  for (i in 1:nrow(Sigma)) {
    for (j in 1:ncol(Sigma)) {
      Sigma[i,j] <- 1+X1[i]*X2[j]
    }
  }
  return(Sigma)
}
 


## Returns the covariance matrix assuming white noise
calcSigmaWhite <- function(X1,X2,tau=1, l=NULL) {
  tausq <- tau^2
  Sigma <- matrix(rep(0, length(X1)*length(X2)), nrow=length(X1))
  for (i in 1:nrow(Sigma)) {
    for (j in 1:ncol(Sigma)) {
      Sigma[i,j] <- tausq*(X1[i]==X2[j])
    }
  }
  return(Sigma)
}

## Returns the covariance matrix assuming brown noise
calcSigmaBrown <- function(X1,X2,tau=1, l=NULL) {
  tausq <- tau^2
  Sigma <- matrix(rep(0, length(X1)*length(X2)), nrow=length(X1))
  for (i in 1:nrow(Sigma)) {
    for (j in 1:ncol(Sigma)) {
      Sigma[i,j] <- tausq*min(X1[i],X2[j])
    }
  }
  return(Sigma)
}
 
## Returns the covariance matrix assuming pink noise
calcSigmaPink <- function(X1,X2,tau=1, l=.5) {
  tausq <- tau^2
  Sigma <- matrix(rep(0, length(X1)*length(X2)), nrow=length(X1))
  for (i in 1:nrow(Sigma)) {
    for (j in 1:ncol(Sigma)) {
      Sigma[i,j] <- .5 * tausq * ( abs(X1[i])^(2*l) + abs(X2[j])^(2*l) - abs(X1[i]-X2[j])^(2*l)  )
    }
  }
  return(Sigma)
}
 
## Returns the covariance matrix assuming a Matern class kernel
calcSigmaMatern <- function(X1,X2,nu=5/2, l=1) {
  library(fields)
  Sigma <- matrix(rep(0, length(X1)*length(X2)), nrow=length(X1))
  for (i in 1:nrow(Sigma)) {
    for (j in 1:ncol(Sigma)) {
      Sigma[i,j] <- Matern(sqrt( (X1[i]-X2[j])^2 ), scale=l, nu=nu)
    }
  }
  return(Sigma)
}

## Returns the covariance matrix assuming a rational quadratic kernel
calcSigmaRQ <- function(X1,X2,alpha=2, l=1) {
  denom <- (2 * alpha * l ^2)
  Sigma <- matrix(rep(0, length(X1)*length(X2)), nrow=length(X1))
  for (i in 1:nrow(Sigma)) {
    for (j in 1:ncol(Sigma)) {
      Sigma[i,j] <- (1+ (X1[i]-X2[j])^2 / denom)^(-alpha)
    }
  }
  return(Sigma)
}

## Returns the covariance matrix assuming a periodic kernel
calcSigmaPeriodic <- function(X1, X2, l=1, p=1) {
  denom <- l ^2
  Sigma <- matrix(rep(0, length(X1)*length(X2)), nrow=length(X1))
  for (i in 1:nrow(Sigma)) {
    for (j in 1:ncol(Sigma)) {
      Sigma[i,j] <- exp(-2 * sin( pi*(X1[i]-X2[j])/p )^2 / denom)
    }
  }
  return(Sigma)

}
