rm(list=ls())
setEPS()
epsPlot <- TRUE

source("gpPlotFuns.R")
source("kernels.R")

## Returns the probit model of a psychometric function 
pmetric <- function(x,  loc = 2, scale = 1, chance = .5, lapserate = .05) {
  chance + (1-lapserate-chance) * pnorm(x, loc, scale)
}

## Samples data from the probit model of a psychometric function 
sampledata <- function(n, x, loc = 2, scale = 1, chance = .5, lapserate = .05) {
  rbinom(1, n, pmetric(x, loc, scale, chance, lapserate))
}

llgpPsychometric <- function (params, x, f.x, tau, nSamples) {
  #tau <- exp(params[1])
  l <- exp(params[1])
  sigma.n <- exp(params[2])
  k.xx <- calcSigmaRBF(x,x, tau, l)
  k.Error <- calcErrorPsychometric(c(0,pc), nSamples, sigma.n)
  -1*(-.5 * t(f.x) %*% jitCholInv(k.xx +k.Error)$invM %*% f.x - .5 * log(det(k.xx +k.Error)) - length(intensities)/2 * log (2 * pi))
}

calcErrorPsychometric <- function(X1, nSamples, sigma.n=0.05) {
  kdiag <- X1 * (1-X1)/nSamples  +sigma.n^2
  kdiag[X1 == 0 ] <- 0
  diag(kdiag)
}




## Set the seed for the random number generator so that the results are replicated
set.seed(1)

#  First, we assume a Weibull psychometric function on a 2AFC task
#   with location=2, scale=.5, and lapserate=.05
location <- 2
scale <- .5
lapserate <- .05


#  Next, we generate data from the model as though the observer were tested
#   using the method of constant stimuli
nSamples <- 50  # Number of trials per level
intensities <- seq(.5,4, by=.5)  #  Levels

nlevels <- length(intensities)  #  Number of levels
pc <- rep(NA, nlevels)  #  Percent correct at each level
for (i in 1:nlevels) {
  pc[i] <- sampledata(nSamples, intensities[i], loc=location, scale=scale, lapserate=lapserate) / nSamples
}

## Plot the psychometric function with the data
xx <- seq(0,5, length.out=100)
if(epsPlot) { postscript("TrueCurve.eps", width=5, height=5) } else {dev.new()}
plot(xx, pmetric(xx, loc=location, scale=scale, lapserate=lapserate), col='blue',
      type='l', xlab="Intensity", ylab="P(correct)", main="True Psychometric Function", ylim=c(.5,1))
points(intensities, pc, pch="+", cex=2)
abline(h=1, lty=2)
text(0.2,.55, "Chance")
arrows(4.5, 1-lapserate, 4.5, 1, angle=90, length=.05, lwd=2, code=3)
text(4.5,.95, "Lapse Rate")
if(epsPlot) {dev.off()}


# Here, we set the range of values of the psychometric function at which we are
#  interested in the posterior distribution
xtest <- seq(0,5, length.out=100)

# We caclulate the covariance matrices assuming a squared exponential kernel

# First attempt, using gptk and default parameters
options = gpOptions()
options$kern$comp = list("rbf", "white")
kern = kernCreate(intensities, list(type = "cmpnd", comp = list("rbf", 
      "white")))

x <- matrix(intensities); y <- matrix(pc)
xtest <- matrix(xtest)

# Estimate the model
model <- gpCreate(dim(x)[2], dim(y)[2], x, y, options)
# Extract the posterior
meanVar <- gpPosteriorMeanVar(model, xtest, varsigma.return = TRUE)

# Plot the results
if(epsPlot) { postscript("Model1.eps", width=10, height=5); par(mfrow=c(1,2)) } else {dev.new()}
gpPlot(model, xtest, meanVar$mu, meanVar$varsigma, ylim = c(.45, 1), xlim = range(xtest), col = "black", title="Default Parameters")
lines(xtest, pmetric(xtest, loc=location, scale=scale, lapserate=lapserate), lwd = 1, col='blue')

# Now, we try again with optimized parameters
optmodel <- gpOptimise(model, display=1)
optmeanVar <- gpPosteriorMeanVar(optmodel, xtest, varsigma.return = TRUE)

if(!epsPlot) {dev.new()}
gpPlot(optmodel, xtest, optmeanVar$mu, optmeanVar$varsigma, ylim = c(.45, 1), xlim = range(xtest), col = "black", title="Optimized Parameters")
lines(xtest, pmetric(xtest, loc=location, scale=scale, lapserate=lapserate), lwd = 1, col='blue')
if(epsPlot) {dev.off()}


# We can add more assumptions to our analysis to improve our predictions
# E.g., in 2AFC, with 0 intensity, the probability of begin correct is .5
x <- matrix(c(0, intensities))
y <- matrix(c(.5, pc))
model <- gpCreate(dim(x)[2], dim(y)[2], x, y, options)
optmodel <- gpOptimise(model, display=1)
optmeanVar <- gpPosteriorMeanVar(optmodel, xtest, varsigma.return = TRUE)

if(epsPlot) { postscript("Model2.eps", width=5, height=5) } else {dev.new()}
gpPlot(optmodel, xtest, optmeanVar$mu, optmeanVar$varsigma, ylim = c(.45, 1), xlim = range(xtest), col = "black")
lines(xtest, pmetric(xtest, loc=location, scale=scale, lapserate=lapserate), lwd = 1, col='blue')
if(epsPlot) {dev.off()}


# Currently the prior is that the psychometric funcition is flat, i.e., mu(t)=0
#  To set a new prior, we can just subtract the new prior mean from the data
# E.g., we can assume a linear prior betwen Pcorrect=0 at 0 intensity and Pcorrect=1 at max intensity
x <- matrix(c(0, intensities))
priory <- x* .5/( max(intensities)+tail(diff(intensities),1)) + .5
priory[priory > 1] <- 1
priorytest <- xtest* .5/( max(intensities)+tail(diff(intensities),1)) + .5
priorytest[priorytest > 1] <- 1
y <- matrix(c(.5, pc) - priory)
model <- gpCreate(dim(x)[2], dim(y)[2], x, y, options)
optmodel <- gpOptimise(model, display=1)
optmeanVar <- gpPosteriorMeanVar(optmodel, xtest, varsigma.return = TRUE)

if(epsPlot) { postscript("Model3.eps", width=5, height=5) } else {dev.new()}
gpPlot(optmodel, xtest, optmeanVar$mu+priorytest, optmeanVar$varsigma, ylim = c(.5, 1), xlim = range(xtest), col = "black")
points(x,y+priory, pch='+')
lines(xtest, pmetric(xtest, loc=location, scale=scale, lapserate=lapserate), lwd = 1, col='blue')
if(epsPlot) {dev.off()}




# We can also do the same by hand, which will give us the option to add in the
#  assumtions that the slope is zero when the intensity is zero or max
#  and to allow the variance of the white noise to depend on the number of samples
#  at each location.
l <-  1.5
tau <- .25
sigma.n <-  0.005
# Calculate covariance matrix among observations
k.xx <- calcSigmaRBF(x,x, tau, l)

# Calculate covariance matrix among observations and test
k.xxs <- calcSigmaRBF(x,xtest, tau, l)
k.xsx <- calcSigmaRBF(xtest,x, tau, l)

# Calculate covariance matrix among test values
k.xsxs <- calcSigmaRBF(xtest,xtest, tau, l)



## Augment observations to require that the slope ends flat
x2 <- matrix(x[length(x)]+tail(diff(x), 1)*2:3)
f.x <- c(y, rep(0, length(x2)) )

kx1prime <- calcSigmaRBF.x1prime(x2, x, tau, l)
kx1sprime <- calcSigmaRBF.x1prime(x2, xtest, tau, l)
kx2sprime <- calcSigmaRBF.x2prime(xtest, x2, tau, l)

kx1x2prime <- calcSigmaRBF.x1x2prime(x2, x2, tau, l)

k2.xx <- rbind(k.xx, kx1prime)
k2.xx <- cbind(k2.xx, rbind(t(kx1prime), kx1x2prime))

k2.xxs <- rbind(k.xxs, kx1sprime)
k2.xsx <- cbind(k.xsx, kx2sprime)


k.Error <- calcErrorPsychometric(c(0,pc), nSamples, sigma.n)
#  Assume error in measurement is 0 for added slope observations
k2.Error <- k.Error
for ( i in 1:length(x2) ) {
  k2.Error <- rbind(k2.Error, rep(0,dim(k2.Error)[1]))
  k2.Error <- cbind(k2.Error, rep(0,dim(k2.Error)[1]))
}


#o1 <- optim(log(c(l, sigma.n)), fn=llgpPsychometric, x=x, f.x=f.x, tau=.25, nSamples=nSamples)
#l <- exp(o1$par[1])
#sigma.n <- exp(o1$par[2])


#k.xx <- cbind(k.xx, kx1prime)
#mu <- k.xsx%*%solve(k.xx + sigma.n^2*diag(1,ncol(k.xx)) )%*%f.x
#Sigma  <- k.xsxs - k.xsx%*%solve(k.xx  + sigma.n^2*diag(1,ncol(k.xx)))%*%k.xxs

mu <- k.xsx%*%solve(k.xx +k.Error)%*%y
Sigma  <- k.xsxs - k.xsx%*%solve(k.xx + k.Error)%*%k.xxs

mu2 <- k2.xsx%*%solve(k2.xx +k2.Error)%*%f.x
Sigma2  <- k.xsxs - k2.xsx%*%solve(k2.xx + k2.Error)%*%k2.xxs


S <- diag(Sigma)
S2 <- diag(Sigma2)


mupc <- mu +priorytest

mu2pc <- mu2 + priorytest

f = c(mupc + 2 * sqrt(abs(S)), rev(mupc - 2 * sqrt(abs(S))))
f2 = c(mu2pc + 2 * sqrt(abs(S2)), rev(mu2pc - 2 * sqrt(abs(S2))))

if(epsPlot) { postscript("Model4.eps", width=5, height=5) } else {dev.new()}
plot(range(xtest), c(0,1), type='n', ylim=c(.45,1), ylab="P(Correct)", xlab="Intensity")
polygon(c(xtest, rev(xtest)), f, col = "gray", border = "gray")
lines(xtest, mupc, lwd = 2)
lines(xtest, pmetric(xx, loc=location, scale=scale, lapserate=lapserate), lwd = 1, col='blue')
points(intensities, pc, pch="+", cex=1)
if(epsPlot) {dev.off()}

if(epsPlot) { postscript("Model5.eps", width=5, height=5) } else {dev.new()}
plot(range(xtest), c(0,1), type='n', ylim=c(.45,1), ylab="P(Correct)", xlab="Intensity")
polygon(c(xtest, rev(xtest)), f2, col = "gray", border = "gray")
lines(xtest, mu2pc, lwd = 2)
lines(xtest, pmetric(xx, loc=location, scale=scale, lapserate=lapserate), lwd = 1, col='blue')

points(intensities, pc, pch="+", cex=1)
if(epsPlot) {dev.off()}

#err <- sqrt(diag(k.Error))[-1]
#arrows(intensities, pc + 2*err, intensities, pc-2*err, code=3, angle=90)

